-------------------------------------------------------------------------------
-- Cloud Serpent by Lockslap, US-Bleeding Hollow
-- http://ba-guild.com
-------------------------------------------------------------------------------
local CloudSerpent		= LibStub("AceAddon-3.0"):NewAddon("Cloud Serpent", "AceEvent-3.0", "AceConsole-3.0")
local L					= LibStub("AceLocale-3.0"):GetLocale(CloudSerpent.name, false)
if not TomTom then return end

CloudSerpent.name		= GetAddOnMetadata("CloudSerpent", "Title")
CloudSerpent.version	= GetAddOnMetadata("CloudSerpent", "Version")
CloudSerpent.db			= {}

local curZone, subZone
local waypoints	= {}
local defaults	= {
	char = {
		enabled = true,
		exalted	= false,
	}
}

-- zones that will activate the waypoints
local zones = {
	[L["Mistveil Sea"]]		= true,
	[L["Windward Isle"]]	= true,
	[L["The Arboretum"]]	= true,
}

local enabledString		= ("|cff00ff00%s|r"):format(L["Enabled"])
local disabledString	= ("|cffff0000%s|r"):format(L["Disabled"])

local function ClearWaypoints()
	if #waypoints == 0 then return end
	for _, point in pairs(waypoints) do
		TomTom:RemoveWaypoint(point)
	end
	waypoints = {}
	TomTom:ReloadWaypoints()
end

local function CheckReputation()
	local _, _, standing, _, _, _, _, _, _, _, _, _, _ = GetFactionInfoByID(1271)
	CloudSerpent.db.char.exalted = standing == 8 and true or false
	return CloudSerpent.db.char.exalted
end

local function ToggleAddon()
	CheckReputation()
	if not CloudSerpent.db.char.exalted then
		CloudSerpent.db.char.enabled = not CloudSerpent.db.char.enabled
		CloudSerpent:Print((L["Waypoints have been %s"]):format(CloudSerpent.db.char.enabled == true and enabledString or disabledString))
		if not CloudSerpent.db.char.enabled and #waypoints > 0 then
			ClearWaypoints()
		elseif CloudSerpent.db.char.enabled == true then
			curZone = GetZoneText()
			subZone = GetSubZoneText()
			CloudSerpent:UpdateWaypoints(curZone, subZone)
		end
	elseif not CloudSerpent.db.char.enabled then
		CloudSerpent.db.char.enabled = false
		self:Print(L["Waypoints automatically disabled because you're exalted with the |cff1784d1Order of the Cloud Serpent|r"])
		if #waypoints > 0 then ClearWaypoints() end
	end
end

function CloudSerpent:UpdateWaypoints(zone, sub)
	if not self.db.char.enabled then return end
	-- only need waypoints in the mistveil sea, windward isle, and the arboretum
	if zone == L["The Jade Forest"] and zones[sub] == true and #waypoints == 0 then
		for _, coord in pairs(self.coordinates) do
			-- add the waypoint to tomtom
			waypoints[#waypoints + 1] = TomTom:AddZWaypoint(6, 6, coord.x, coord.y, L["Onyx Egg Location"], false, false, false, false, false, true)
		end
		--self:Print((L["Loaded |cff1784d1%d|r waypoints."]):format(#self.coordinates))
	elseif zone ~= L["The Jade Forest"] or not zones[sub] then
		ClearWaypoints()
	end
end

function CloudSerpent:HandleEvents(event, ...)
	-- these will need to update regardless of which event fired
	curZone = GetZoneText()
	subZone = GetSubZoneText()
	
	if event == "PLAYER_ENTERING_WORLD" then
		self.db = LibStub("AceDB-3.0"):New("CloudSerpentDB", defaults)
		
		-- check reputation
		CheckReputation()
		
		if self.db.char.exalted == true and self.db.char.enabled == true then
			self.db.char.enabled = false
			self:Print(L["Waypoints automatically disabled because you're exalted with the |cff1784d1Order of the Cloud Serpent|r"])
		end
		
		-- set tomtom's arrival distance to 5
		TomTom.profile.arrow.arrival = 5
		
		-- register the chat commands to toggle the addon
		self:RegisterChatCommand("cloudserpent", function() ToggleAddon() end)
		self:RegisterChatCommand("cs", function() ToggleAddon() end)
		
		self:UnregisterEvent("PLAYER_ENTERING_WORLD")
	end
	
	if self.db.char.enabled == true and not self.db.char.exalted then
		self:UpdateWaypoints(curZone, subZone)
	end
end

function CloudSerpent:CHAT_MSG_COMBAT_FACTION_CHANGE()
	-- check if exalted with the order
	CheckReputation()
	if self.db.char.exalted == true then
		-- exalted, so disable waypoints
		self.db.char.enabled = false
		self:Print(L["Waypoints automatically disabled because you're exalted with the |cff1784d1Order of the Cloud Serpent|r"])
		if #waypoints > 0 then ClearWaypoints() end
	end
end

-- tracked events
CloudSerpent:RegisterEvent("ZONE_CHANGED", "HandleEvents")
CloudSerpent:RegisterEvent("ZONE_CHANGED_INDOORS", "HandleEvents")
CloudSerpent:RegisterEvent("ZONE_CHANGED_NEW_AREA", "HandleEvents")
CloudSerpent:RegisterEvent("PLAYER_ENTERING_WORLD", "HandleEvents")
CloudSerpent:RegisterEvent("CHAT_MSG_COMBAT_FACTION_CHANGE")

-- the coordinates of known egg locations
CloudSerpent.coordinates = {
	{x = 65.08, y = 26.20},
	{x = 66.64, y = 28.69},
	{x = 67.12, y = 24.94},
	{x = 65.18, y = 24.77},
	{x = 67.05, y = 33.81},
	{x = 66.61, y = 31.58},
	{x = 66.54, y = 32.20},
	{x = 63.09, y = 29.31},
	{x = 69.65, y = 31.64},
	{x = 70.53, y = 28.99},
	{x = 61.89, y = 30.20},
	{x = 66.18, y = 33.43},
	{x = 62.40, y = 32.36},
	{x = 65.63, y = 25.07},
	{x = 64.18, y = 23.92},
	{x = 63.39, y = 21.62},
	{x = 62.98, y = 21.95},
	{x = 62.37, y = 32.84},
	{x = 62.17, y = 31.75},
	{x = 62.19, y = 31.21},
	{x = 67.02, y = 32.68},
	{x = 62.46, y = 28.77},
	{x = 62.21, y = 29.29},
	{x = 62.00, y = 29.54},
	{x = 63.73, y = 21.90},
	{x = 62.30, y = 28.12},
	{x = 62.71, y = 29.78},
	{x = 66.43, y = 29.55},
	{x = 65.18, y = 24.77},
	{x = 65.24, y = 23.91},
	{x = 70.33, y = 28.63},
	{x = 67.06, y = 30.25},
}